var express = require('express');
var url = require('url');
var app = express();
var process = require('process');
var request = require('request');
var exphbs  = require('express-handlebars');
var bodyParser = require("body-parser");
var config = require(__dirname + '/config.json');
var fs = require('fs');
var path = require('path');
const dateTime = require('date-and-time')

var logrotate = require('logrotator');
var rotator = logrotate.rotator;

var imageBase64Code;

rotator.register('/var/log/artix/artixcs-sale-renderer/artixcs-sale-renderer.log', {
  schedule: '5m',
  size: '10m',
  compress: true,
  count: 3,
  format: function(index) {
    var d = new Date();
    return d.getDate()+"-"+d.getMonth()+"-"+d.getFullYear();
  }
});


// Настройка хелперов handlebars. Хелперы позволяют выполнить какие-то действия с данными
// на уровне шаблона.

var hbs = exphbs.create({
  helpers: {
    'getTime': function(data) {
      var d = new Date(data);
      return dateTime.format(d, 'HH:mm')
    },
    'getDate' : function(date) {
      var d = new Date(date);
      return dateTime.format(d, "DD-MM-YYYY");
    },
    'calcDiscountSum' : function(data) {
      var total =  0.00;
      if (data.docType == 1 || data.docType == 2 || data.docType == 3) {
      for(var i=0;i<data.inventPositions.length;i++)
      {
        total += parseFloat(data.inventPositions[i].sume.toFixed(2));
      }
      return (total.toFixed(2) - parseFloat(data.docSum)).toFixed(2);
      }
      return 0;
    },
    'getDiscountMode' : function(data) {
      if (data.discType == "percent") {
        return "Сумма скидки " + data.discSize + "%";
      } else if (data.discType == "summ") {
        return "Сумма скидки";
      } else if (data.discType == "price") {
        return "Специальная цена"
      } else if (data.discType == "bonus") {
        return "Скидка по бонусам " + data.discSize;
      } else if (data.discType == "index") {
        return "Индексная скидка"
      } else {
        return "Скидка";
      }
    },
    'getDocumentType' : function(data) {
      if (data.docType == "1") {
        return "Продажа";
      } else if (data.docType == "2") {
        return "Возврат";
      } else if (data.docType == "3") {
        return "Внесение"
      } else if (data.docType == "4") {
        return "Выем";
      } else if (data.docType == "7") {
        return "Аннулирование";
      } else if (data.docType == "13"){
        return "Остаток денег на начало смены";
      } else if (data.docType == "16") {
        return "Документ инвентаризации";
      } else if (data.docType == "18") {
        return "Возврат поставщику";
      } else if (data.docType == "25") {
        return "Возврат по чеку продажи";
      } else if (data.docType == "26") {
        return "Чек коррекции прихода";
      } else if (data.docType == "27") {
        return "Чек коррекции расхода";
      } else {
        return "";
      }
    },
    'isNotEmpty' : function(data) {
      return data != null && data.length > 0;
    },
    'isNeedDiscount' : function(data) {
      return !(data == 3 || data == 4)
    },
    'base64image' :  function() {
        fs.readFile(path.join(__dirname + '/css/picts/logo1.png'), function (error, data) {
            if (error) {
                 throw error;
            } else {
                  imageBase64Code = 'data:image/png;base64,' +  Buffer.from(data).toString('base64');
            }
        });
     return imageBase64Code;
     }
  }
});

app.engine('handlebars', hbs.engine);
app.set('view engine', 'handlebars');
app.use(bodyParser.json());

var data = {
  CS_SERVER_HOST: config.CS_SERVER_HOST,
  CS_SERVER_PORT: config.CS_SERVER_PORT,
  SR_PORT: config.SR_PORT,
  CS_USER_NAME: config.CS_USER_NAME,
  CS_USER_PASSWORD: config.CS_USER_PASSWORD
};

// routing
// выдача чека по идентификатору
app.get('/v1/getReceiptAsImage', function(req, res) {
  var url_parts = url.parse(req.url, true);
  var query = url_parts.query;
  if (query.id === undefined) {
    res.send("Не указан идентификатор чека (поле id)");
  }
  var csUrl = 'http://' + data.CS_SERVER_HOST +  ':' + data.CS_SERVER_PORT +  '/CSrest/rest/sales/document/' + query.id+ '?shopId=' + query.storeId + '&idType=5' + ((query.clusterId === null || query.clusterId === undefined) ? '' : '&clusterId=' + query.clusterId);
  log(csUrl);
  request(getAuthHeader(csUrl), function (error, response, body) {
    if (!error && response.statusCode == 200) {
      var receipt = JSON.parse(body);
      var toTemplate = {'receipt': receipt[0]};
      res.render(__dirname + '/views/layouts/main.handlebars', toTemplate);
    } else if (!error && response.statusCode == 403) {
      res.send("Ошибка авторизации");
    } else if (error) {
      res.send("Произошла ошибка при запросе чека с кассового сервера");
    } else {
      res.send("Ошибка. Код: " + response.statusCode);
    }
  });
});

app.get('/v1/getReceiptByTransactionData', function(req, res) {
  var url_parts = url.parse(req.url, true);
  var query = url_parts.query;
  var csUrl = 'http://' + data.CS_SERVER_HOST +  ':' + data.CS_SERVER_PORT +  '/CSrest/rest/sales/document/find' + '?shopId=' + query.shopId + '&cashCode=' + query.cashCode + '&shiftNum=' + query.shiftNum + '&checkNum=' + query.checkNum + "&timeFromCash=" + query.timeFromCash;
  log(csUrl);
  request(getAuthHeader(csUrl), function (error, response, body) {
    if (!error && response.statusCode == 200) {
      var receipt = JSON.parse(body);
      var toTemplate = {'receipt': receipt[0]};
      res.render(__dirname + '/views/layouts/main.handlebars', toTemplate);
    } else if (!error && response.statusCode == 403){
      res.send("Ошибка авторизации");
    } else if (error) {
      res.send("Произошла ошибка при запросе чека с кассового сервера");
    } else {
      res.send("Ошибка. Код: " + response.statusCode);
    }
  });
});

app.post('/v1/renderReceiptAsImage', function(req, res) {
  log("Построение документа для: ");
  log(req.body);
  if (!req.body) return res.sendStatus(400);
  res.render(__dirname + '/views/layouts/main.handlebars', {'receipt' : req.body});
});

app.use("/v1/css", express.static(__dirname + "/css"));
app.use(express.static('css'));
app.listen(data.SR_PORT);
module.exports = app;

function log(text) {
    console.log("[" + new Date().toISOString() + "] " + text);
}

function getAuthHeader(url) {
    if (Boolean(data.CS_USER_NAME && data.CS_USER_PASSWORD)) {
        return {
            url: url,
            headers: {"Authorization": "Basic " + new Buffer.from(data.CS_USER_NAME + ":" + data.CS_USER_PASSWORD).toString("base64")}
        }
    }
    return {url: url}
}